#include <Conceptinetics.h>
#include <Rdm_Defines.h>
#include <Rdm_Uid.h>
#include <SPI.h>
#include <Ethernet.h>

// replace the MAC address below with the MAC address on your Arduino ethernet device
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };

EthernetClient client;

#define DMX_MASTER_CHANNELS   100
#define RXEN_PIN                2
#define DELIMITER             '!'
#define NULL_CHAR             '\0'
#define FETCH_TIMEOUT         2000
#define LOW_SHORT             '#'
#define HIGH_SHORT            '^'
#define BYTE_LOW              0x00
#define BYTE_HIGH             0xff
#define LOOP_RATE             100
#define LOOP_WAIT_MS          (int)(1000/LOOP_RATE)
DMX_Master        dmx_master ( DMX_MASTER_CHANNELS, RXEN_PIN );
int    HTTP_PORT   = 80;
String inputString  = "";
String compareString = "";
char   client_input = NULL_CHAR;
boolean webString = false;
String HTTP_METHOD = "GET /";
String Username    = "";
String Password    = "";
String Api_Key     = "";
String Returntype  = "string"; // string or array
char   HOST_NAME[] = "www.stagemonsterlive.com";
String PATH_NAME   = "access?username=" + Username +
                     "&password=" + Password + "&apikey=" + Api_Key + "&returntype=" + Returntype;


void setup() {
    dmx_master.enable ();
    dmx_master.setChannelRange ( 1, 25, 127 );
    for (int x = 1; x <= 512; x++) {
        dmx_master.setChannelValue(x, 000);
    }
    Serial.begin(115200);

    if (Ethernet.begin(mac) == 0) {
        Serial.println("Failed to obtain an IP address using DHCP");
        while(true);
    }
}
void getData(){
    if ( !serverConnect() ){
  //    Serial.println("WARNING: Server not responding...");
        serverDisconnect();
        return;
    }
    String inputString = "";
    client_input = NULL_CHAR;
    bool string_started = 0;
    for (;;){
        if ( client_input == DELIMITER ){
            if ( string_started ) break;
            else string_started = 1;
        }
        else if ( string_started )
        inputString += client_input;
        unsigned long timestamp = millis();
        while ( !client.available() ){
            if ( ( millis() - timestamp ) > FETCH_TIMEOUT ){
                serverDisconnect();
                return;
            }
        }
        client_input = client.read();
    }
    /* Ensure new string is at least min length */
    if ( inputString.length() < 6 ){
        serverDisconnect();
        return;
    }
/* Ensure new string is different from last */
    if ( inputString.substring(0, 10) != compareString ){
        Serial.println("change");
        webString = true;
  /* Extract integers from input */
        int outb = (inputString.substring(10, 13)).toInt();
        int drb  = (inputString.substring(13, 16)).toInt();

  /* Update DMX channels with input string  */
        if ( !updateDMXChannels( inputString.substring(10), 511 ) ){
//      Serial.println("WARNING: Incomplete input string.");
    /* Disconnect from server */
            serverDisconnect();
        }

  /* Update last two channels with outb and drb if using expansion modules*/
        dmx_master.setChannelValue(511, byteValueCheck(outb));
        dmx_master.setChannelValue(512, byteValueCheck(drb ));

  /* Set save current string as compare string */
        compareString = inputString.substring(0, 10);

  /* Disconnect from server */
        serverDisconnect();
    }
    else{
  /* Disconnect from server */
        serverDisconnect();
    }

/* Wait before looping */
    delay(LOOP_WAIT_MS);

}
bool updateDMXChannels( String in, int n ){
  /* Assume input string is 1-valid */
    bool ret = 1;

  /* Update first N channels where N is input length */
    int index = 6, channel = 1, input_length = (int)in.length();
    for ( ; index < input_length && channel <= n; channel++ ){
        String curr = in.substring(index, index + 1);
        switch ( curr.charAt(0) ){
            case LOW_SHORT:
                dmx_master.setChannelValue( channel, BYTE_LOW );
                index ++;
                break;
            case HIGH_SHORT:
                dmx_master.setChannelValue( channel, BYTE_HIGH );
                index ++;
                break;
            default:
        /* Ensure there are 3 values to parse */
                if ( index + 2 < input_length ){
                    curr += in.substring(index + 1, index + 3);
                    byte value = byteValueCheck(curr.toInt());
                    dmx_master.setChannelValue(channel, value);
                    index += 3;
                }
        /* If the input string is incomplete clear current
           channel and set return to 0-warning */
                else{
                    dmx_master.setChannelValue( channel, BYTE_LOW );
                    ret = 0;
                }
        }
    }

  /* Zero remaining channels */
    for ( ; channel <= n; channel++ )
        dmx_master.setChannelValue( channel, BYTE_LOW );

        return ret;
}
byte byteValueCheck( int value ){
    if      ( value < 0 )         return BYTE_LOW;
    else if ( value > BYTE_HIGH ) return BYTE_HIGH;
    else                          return (byte)value;
}
bool serverConnect( void ){
  /* Try connection N times where N is max connection attempts */
    for ( int i = 1; i <= 3; i++ ){
        if ( client.connect(HOST_NAME, HTTP_PORT) ){
            /* Break loop when connected */
            Serial.println("Connected to server");
            client.println(HTTP_METHOD + PATH_NAME + " HTTP/1.1");
            client.println("Host: " + String(HOST_NAME));
            client.println("Connection: close");
            client.println();
            return 1;
        }
    }
    return 0;
}

void serverDisconnect( void ){
    client.stop();
}
void loop() {
    getData();
}
